( cd usr/bin ; rm -rf aclocal )
( cd usr/bin ; ln -sf aclocal-1.8 aclocal )
( cd usr/bin ; rm -rf automake )
( cd usr/bin ; ln -sf automake-1.8 automake )
# Install the info files for this package
if [ -x usr/bin/install-info ]
then
   usr/bin/install-info --info-dir=/usr/info /usr/info/automake.info.gz 2>/dev/null
fi

